// OpenCP Module Player
// copyright (c) '94-'98 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
//
// CPIFace background picture loader
//
// revision history: (please note changes here)
//  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
//    -first release
//  -kb980717   Tammo Hinrichs <opencp@gmx.net>
//    -some minor changes
//  -doj980928  Dirk Jagdmann <doj@cubic.org>
//    -wrote a more compatible TGA loader which is found in the file tga.cpp
//    -changed this file to meet dependencies from the new TGA loader
//    -added comments
//  -doj980930  Dirk Jagdmann <doj@cubic.org>
//    -added gif loader
//  -doj981105  Dirk Jagdmann <doj@cubic.org>
//    -now the plOpenCPPict is cleared if no file is found / valid
//    -modified memory allocation a bit to remove unnecessary new/delete
//  -fd981119   Felix Domke <tmbinc@gmx.net>
//    -added the really important 'NO_CPIFACE_IMPORT'
//  -doj20020902 Dirk Jagdmann <doj@cubic.org>
//    -added wildcard for picture loading

#define NO_CPIFACE_IMPORT
#ifdef __WATCOMC__
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <direct.h>
#endif
#ifndef __WATCOMC__
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#endif

#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "psetting.h"
#include "cpiface.h"

#include "gif.h"
#include "tga.h"

#include "pmain.h"

unsigned char *plOpenCPPict=0; // an array containing the raw picture
unsigned char plOpenCPPal[768]; // the palette for the picture
extern char cfDataDir[]; // directory where pictures are stored

static const int picWidth=640;
static const int picHeight=384;

struct node_t {
  char *name;
  node_t *next;
};

node_t *files=0;
int filesSize=0;

static int match(const char *name)
{
  // check file suffixes
  int l=strlen(name);
  if(l<5) return 0;
  if(name[l-4]!='.') return 0;
  if(tolower(name[l-3])!='g' && tolower(name[l-3])!='t') return 0;
  if(tolower(name[l-2])!='i' && tolower(name[l-2])!='g') return 0;
  if(tolower(name[l-1])!='f' && tolower(name[l-1])!='a') return 0;

  return 1;
}

void plReadOpenCPPic()
{
  int i;
  node_t* nd;

  static int lastN=-1;
  // setup file names
  if(lastN==-1)
    {
      node_t* *current=&files;
      // get the resource containing background pictures
      const char *picstr=cfGetProfileString2(cfScreenSec, "screen", "usepics", "");

      int wildcardflag=0;

      // n contains the number of background pictures
      int n=cfCountSpaceList(picstr, 12);
      for(i=0; i<n; i++)
	{
	  char name[_MAX_PATH];
	  if(cfGetSpaceListEntry(name, picstr, 12) == 0) break;
	  if(!match(name)) continue;
	  // check for wildcard
	  if(!strnicmp(name, "*.gif", 5) || !strnicmp(name, "*.tga", 5))
	    {
	      if(wildcardflag)
		continue;
	      DIR *dir=opendir(cfDataDir);
	      if(dir)
		{
		  while(1)
		    {
		      struct dirent *dp=readdir(dir);
		      if(dp==0) break;
		      if(match(dp->d_name))
			{
			  nd=new node_t;
			  nd->name=strdup(dp->d_name);
			  nd->next=0;
			  *current=nd;
			  current=&nd->next;
			  filesSize++;
			}
		    }
		  closedir(dir);
		}
	      wildcardflag=1;
	    }
	  else
	    {
	      nd=new node_t;
	      nd->name=strdup(name);
	      nd->next=0;
	      *current=nd;
	      current=&nd->next;
	      filesSize++;
	    }
	}      

#if 0
      DEBUGSTR("images:");
      nd=files;
      while(nd)
	{
	  DEBUGSTR(nd->name);
	  nd=nd->next;
	}
#endif

    }

  // if there are no background pictures we can skip the rest
  if (filesSize<=0)
    return;
  // choose a new picture
  int n=rand()%filesSize;
  // we have loaded that picture already
  if(n==lastN)
    return; 
  lastN=n;

  // get filename
  nd=files;
  for(i=0; i<n; i++)
    nd=files->next;

  char path[_MAX_PATH];
  sprintf(path, "%s%s", cfDataDir, nd->name);

  // read the file into a filecache
  int file=open(path, O_RDONLY|O_BINARY);
  if(file<0)
    return;
  int filesize=lseek(file, 0, SEEK_END);
  if(filesize<0)
    {
      close(file);
      return ;
    }
  if(lseek(file, 0, SEEK_SET)<0)
    {
      close(file);
      return ;
    }
  unsigned char *filecache=new unsigned char[filesize];
  if(filecache==0) 
    {
      close(file);
      return ;
    }
  if(read(file, filecache, filesize)!=filesize) 
    {
      delete [] filecache;
      close(file);
      return ;
    }
  close(file);

  // allocate memory for a new picture
  if(plOpenCPPict==0)
    {
      plOpenCPPict=new unsigned char [picWidth*picHeight];
      if(plOpenCPPict==0) return;
      memset(plOpenCPPict, 0, picWidth*picHeight);
    }

  // read the picture
  GIF87read(filecache, filesize, plOpenCPPict, plOpenCPPal, picWidth, picHeight);
  TGAread(filecache, filesize, plOpenCPPict, plOpenCPPal, picWidth, picHeight);

  delete [] filecache;

  // determine if the lower or upper part of the palette is left blank for
  // our own screen colors
  int low=0;
  int high=0;

  for (i=0; i<picWidth*picHeight; i++)
    if (plOpenCPPict[i]<0x30)
      low=1;
    else if (plOpenCPPict[i]>=0xD0)
      high=1;

  int move=(low&&!high)*0x90;

  // if the color indices are bad, we have to move every color map entry
  if (move)
    for (i=0; i<picWidth*picHeight; i++)
      plOpenCPPict[i]+=0x30;

  // adjust the RGB palette to 6bit, because standard VGA is limited
  for (i=0x2FD; i>=0x90; i--)
    plOpenCPPal[i]=plOpenCPPal[i-move]>>2;
}
